#ifndef __MATERIAL_H
#define __MATERIAL_H

#include "glInterface.h"
#include "common.h"


namespace CHCDemoEngine 
{

class Texture;
class RenderState;


class RgbaColor 
{

public:

	float r, g, b, a;

	RgbaColor(): r(1), g(1), b(1), a(1)
	{}

	RgbaColor(float _r, float _g, float _b, float _a): r(_r), g(_g), b(_b), a(_a)
	{}

	friend RgbaColor RandomColor(float a = 0.0f, float b = 1.0f);
};


// Forward declarations
RgbaColor RandomColor(float a, float b);



class Material 
{
friend class ResourceManager;

public:

	/** Sets default material (ambient intensity 0.2f, diffuse intensity 1.0f)
	*/
	Material();

	/** Sets ambient and diffuse color to color
	*/
	Material(const RgbaColor &color);

	friend Material RandomMaterial();

	inline Texture *GetTexture() const { return mTexture; }

	inline RgbaColor GetAmbient() const { return mAmbientColor; }
	inline RgbaColor GetDiffuse() const { return mDiffuseColor; }
	inline RgbaColor GetSpecular() const { return mSpecularColor; }
	inline RgbaColor GetEmmisive() const { return mEmmisiveColor; }

	inline void SetTexture(Texture *texture) { mTexture = texture; }

	inline void SetAmbient(const RgbaColor &color) { mAmbientColor = color; }
	inline void SetDiffuse(const RgbaColor &color) { mDiffuseColor = color; }
	inline void SetSpecular(const RgbaColor &color) { mSpecularColor = color; }
	inline void SetEmmisive(const RgbaColor &color) { mEmmisiveColor = color; }
	
	inline void SetAlphaTestEnabled(bool alpha) { mAlphaTestEnabled = alpha; }
	inline void SetCullFaceEnabled(bool cull) { mCullFaceEnabled = cull; }

	inline bool IsAlphaTestEnabled() const { return mAlphaTestEnabled; } 
	inline bool IsCullFaceEnabled() const { return mCullFaceEnabled; } 

	/** Renders this material.
	*/
	void Render(RenderState *state);
	/** Initialize the material with default values
	*/
	void InitMaterial();

protected:

	///////////

	RgbaColor mDiffuseColor;
	RgbaColor mSpecularColor;
	RgbaColor mAmbientColor;
	RgbaColor mEmmisiveColor;

	bool mAlphaTestEnabled;
	bool mCullFaceEnabled;
	/// the associated texture
	Texture *mTexture;
};


extern Material RandomMaterial();

}

#endif
